from copy import deepcopy
from typing import Dict, Tuple, TypeVar

import click

from anyscale._private.workload import WorkloadConfig


class AnyscaleCommand(click.Command):
    """
    AnyscaleCommand is a subclass of click.Command that allows for the addition of
    CLI examples.

    :param __doc_cli_example__: A string that represents CLI examples.
    """

    def __init__(self, *args, **kwargs):
        self.__doc_cli_example__ = kwargs.pop("example", None)
        self.__is_alpha__ = kwargs.pop("is_alpha", False)
        self.__is_beta__ = kwargs.pop("is_beta", False)
        super().__init__(*args, **kwargs)

    @property
    def is_alpha(self) -> bool:
        return self.__is_alpha__

    @property
    def is_beta(self) -> bool:
        return self.__is_beta__


class LegacyAnyscaleCommand(click.Command):
    """
    LegacyAnyscaleCommand is a subclass of click.Command that allows for the addition of
    upgrade information.

    :param __doc_new_cli__: The CLI command that the legacy command should be upgraded to.
    :param __doc_new_prefix__ : The prefix of the new CLI command.
    """

    def __init__(self, *args, **kwargs):
        self.__doc_new_cli__ = kwargs.pop("new_cli", None)
        self.__doc_new_prefix__ = kwargs.pop("new_prefix", None)
        self.__doc_legacy_prefix__ = kwargs.pop("legacy_prefix", None)
        self.__is_limited_support__ = kwargs.pop("is_limited_support", False)
        super().__init__(*args, **kwargs)

    def get_new_cli(self) -> AnyscaleCommand:
        return self.__doc_new_cli__

    def get_new_prefix(self) -> str:
        return self.__doc_new_prefix__

    def get_legacy_prefix(self) -> str:
        return self.__doc_legacy_prefix__

    def is_limited_support(self) -> bool:
        return self.__is_limited_support__


# Take from https://stackoverflow.com/questions/51846634/click-dynamic-defaults-for-prompts-based-on-other-options
class OptionPromptNull(click.Option):
    """
    Option class that allows default values based on previous params
    """

    _value_key = "_default_val"

    def __init__(self, *args, **kwargs):
        self.default_option = kwargs.pop("default_option", None)
        super().__init__(*args, **kwargs)

    def get_default(self, ctx, **kwargs):
        if not hasattr(self, self._value_key):
            if self.default_option is None:
                default = super().get_default(ctx, **kwargs)
            else:
                arg = ctx.params.get(self.default_option)
                if arg is not None:
                    default = self.type_cast_value(ctx, self.default(arg))
                else:
                    default = None
            setattr(self, self._value_key, default)
        return getattr(self, self._value_key)


# Taken from https://stackoverflow.com/questions/44247099/click-command-line-interfaces-make-options-required-if-other-optional-option-is
class NotRequiredIf(click.Option):
    """
    Option class that allows an option to be not required if a separate option is available.
    """

    def __init__(self, *args, **kwargs):
        self.not_required_if = kwargs.pop("not_required_if")
        assert self.not_required_if, "'not_required_if' parameter required"
        kwargs["help"] = (
            kwargs.get("help", "")
            + " NOTE: This argument is mutually exclusive with %s"
            % self.not_required_if
        ).strip()
        super().__init__(*args, **kwargs)

    def handle_parse_result(self, ctx, opts, args):
        we_are_present = self.name in opts
        other_present = self.not_required_if in opts

        if other_present:
            if we_are_present:
                raise click.UsageError(
                    "Illegal usage: `%s` is mutually exclusive with `%s`"
                    % (self.name, self.not_required_if)
                )
            else:
                self.prompt = None
                self.default = None

        return super().handle_parse_result(ctx, opts, args)


def convert_kv_strings_to_dict(strings: Tuple[str]) -> Dict[str, str]:
    """Convert args/env_vars of the form "key=val" into a dictionary of {key: val}.

    NOTE(edoakes): this mimics the functionality of the `serve run` CLI and should be
    kept in sync with it.
    """
    ret_dict = {}
    for s in strings:
        split = s.split("=", maxsplit=1)
        if len(split) != 2 or len(split[1]) == 0:
            raise click.ClickException(
                f"Invalid key-value string '{s}'. Must be of the form 'key=value'."
            )

        ret_dict[split[0]] = split[1]

    return ret_dict


T = TypeVar("T", bound=WorkloadConfig)


def override_env_vars(config: T, overrides: Dict[str, str]) -> T:
    """Returns a new copy of the WorkloadConfig with env vars overridden.

    This is a per-key override, so keys already in the config that are not specified in
    overrides will not be updated.
    """
    if not overrides:
        return config

    final_env_vars = deepcopy(config.env_vars) if config.env_vars else {}
    final_env_vars.update(overrides)
    return config.options(env_vars=final_env_vars)



import contextlib
from contextlib import contextmanager
import os
import sys
import time
from typing import List, Optional

from botocore.exceptions import ClientError
import click
import colorama

from anyscale.anyscale_halo import Halo
from anyscale.client.openapi_client.models import (
    CloudAnalyticsEventCloudProviderError,
    CloudAnalyticsEventCloudResource,
)
from anyscale.utils.imports.gcp import try_import_gcp_exceptions


_process_start_time = time.time()


def bold(text: str, color: Optional[int] = None) -> str:
    """Outputs the text into ANSI-supported bolded text.

    `colorama.Style.BRIGHT` corresponds to bold"""
    return f"{colorama.Style.BRIGHT}{color if color else ''}{text}{colorama.Style.RESET_ALL}"


# 2 is used for consistency across other indenting / padding in CLI
INDENT = " " * 2


def pad_string(text: str, padding: int = 10) -> str:
    """Pads the remainder of text with spaces (`spaces == padding - len(text)`)

    NOTE: `padding` should be longer than `len(text)` for this to be useful"""
    return f"{text: <{padding}}"


class BlockLogger:
    """
    Logger class for the CLI. Supports formatting in blocks if `block_label` is provided to
    the methods. Also supports anyscale.connect style logging if `block_label` is not
    provided.
    """

    def __init__(
        self,
        log_output: bool = True,
        t0: float = _process_start_time,
        spinner_manager: Optional[Halo] = None,
    ) -> None:
        self.t0 = t0
        # Flag to disable all terminal output from CLILogger (useful for SDK)
        self.log_output = log_output
        self.current_block: Optional[str] = None
        self.spinner_manager = spinner_manager
        self.indent_level: int = 0

    def open_block(
        self, block_label: str, block_title: Optional[str] = None, auto_close=False
    ) -> None:
        """
        Prints block title from the given block_label and sets self.current_block. "Output"
        is a generic block that does not need to follow the standard convention.

        if auto_close is set, automatically closes the block before opening a new one
        """
        if not self.log_output:
            return
        assert (
            auto_close or self.current_block is None or self.current_block == "Output"
        ), f"Block {self.current_block} is already open. Please close before opening block {block_label}."

        if auto_close:
            self.close_block()
        self.current_block = block_label
        print(
            f"{colorama.Style.BRIGHT}{colorama.Fore.CYAN}{block_title if block_title else block_label}{colorama.Style.RESET_ALL}",
            file=sys.stderr,
        )

    def close_block(self, block_label: Optional[str] = None) -> None:
        """ Closes the current block
        If a label is specified, it must match the current open block label.
        raise an AssertionError if we try to close a different block

        Prints newline so there is separation before next block is opened.
        """
        if not self.log_output:
            return
        if block_label:
            assert (
                self.current_block == block_label
            ), f"Attempting to close block {block_label}, but block {self.current_block} is currently open."
        self.current_block = None
        print(file=sys.stderr)

    @staticmethod
    def highlight(text: str) -> str:
        return bold(text, colorama.Fore.MAGENTA)

    def zero_time(self) -> None:
        self.t0 = time.time()

    def info(
        self, *msg: str, block_label: Optional[str] = None, end: str = "\n"
    ) -> None:
        if not self.log_output:
            return
        if block_label:
            # Check block_label if provided.
            assert (
                self.current_block == block_label
            ), f"Attempting to log to block {block_label}, but block {self.current_block} is currently open."
            print(INDENT * self.indent_level, end="", file=sys.stderr)
            print(
                *msg, file=sys.stderr,
            )
        else:
            print(
                "{}{}(anyscale +{}){} ".format(
                    colorama.Style.BRIGHT,
                    colorama.Fore.CYAN,
                    self._time_string(),
                    colorama.Style.RESET_ALL,
                ),
                end="",
                file=sys.stderr,
            )
            print(INDENT * self.indent_level, end="", file=sys.stderr)
            print(
                *msg, file=sys.stderr, end=end,
            )

    def debug(self, *msg: str) -> None:
        if not self.log_output:
            return
        if os.environ.get("ANYSCALE_DEBUG") == "1":
            print(
                "{}{}(anyscale +{}){} ".format(
                    colorama.Style.DIM,
                    colorama.Fore.CYAN,
                    self._time_string(),
                    colorama.Style.RESET_ALL,
                ),
                end="",
            )
            print(INDENT * self.indent_level, end="", file=sys.stderr)
            print(*msg)

    def warning(self, *msg: str) -> None:
        if not self.log_output:
            return
        print(
            "{}{}[Warning]{} ".format(
                colorama.Style.NORMAL, colorama.Fore.YELLOW, colorama.Style.RESET_ALL,
            ),
            end="",
            file=sys.stderr,
        )
        print(*msg, file=sys.stderr)

    def _time_string(self) -> str:
        delta = time.time() - self.t0
        hours = 0
        minutes = 0
        while delta > 3600:
            hours += 1
            delta -= 3600
        while delta > 60:
            minutes += 1
            delta -= 60
        output = ""
        if hours:
            output += f"{hours}h"
        if minutes:
            output += f"{minutes}m"
        output += f"{round(delta, 1)}s"
        return output

    def error(self, *msg: str) -> None:
        prefix_msg = f"(anyscale +{self._time_string()})"
        self.print_red_error_message(prefix_msg, end_char="")

        print(*msg, file=sys.stderr)

    def confirm_missing_permission(self, msg):
        if self.spinner_manager:
            self.spinner_manager.stop()

        self.warning(msg)

        self.print_red_error_message(
            "[DANGER] To continue without these permissions press 'y', or press 'N' to abort.",
            end_char="",
        )

        click.confirm(
            "", abort=True,
        )

        if self.spinner_manager:
            self.spinner_manager.start()

    def print_red_error_message(self, error_msg, end_char="\n"):
        print(
            "{}{}{} {}".format(
                colorama.Style.BRIGHT,
                colorama.Fore.RED,
                error_msg,
                colorama.Style.RESET_ALL,
            ),
            end=end_char,
            file=sys.stderr,
        )

    @contextmanager
    def indent(self):
        """ Indent all output within the context
        """
        try:
            self.indent_level += 1
            yield
        finally:
            self.indent_level -= 1

    @contextmanager
    def spinner(self, msg: str):
        """DEPRECATED:

        See https://github.com/manrajgrover/halo/issues/179.
        This `Halo` spinner is not compatible with newer Python notebook versions.
        (iPython and Jupyter)

        Use `rich.spinner.Spinner` or `rich.progress.Progress` instead.

        ---
        Create a spinner with the starting text.

        To update the text next to the spinner, set `spinner.text = "new_text"`
        The spinner will be stopped when the context is exited.
        Spinner is only enabled for interactive shell
        """

        enable_spinner = self.is_interactive_cli_enabled()

        with Halo(
            text=msg, spinner="dots", stream=sys.stderr, enabled=enable_spinner
        ) as spinner:
            yield spinner

    def is_interactive_cli_enabled(self) -> bool:
        """Check if shell is interactive
        """

        default_env_var = "1" if sys.stderr.isatty() else "0"
        return os.environ.get("ANYSCALE_CLI_INTERACTIVE_UX", default_env_var) == "1"


class LogsLogger(BlockLogger):
    """ This logger is used to print customer logs to STDOUT with no decoration
    """

    def log(self, msg: str):
        print(msg)


class StringLogger(LogsLogger):
    """ This logger is used to print customer logs to a string with no decoration
    """

    def __init__(self):
        super().__init__(False)
        self.out_string = ""

    def log(self, msg: str):
        self.out_string += f"{msg}\n"

    def is_interactive_cli_enabled(self) -> bool:
        """Check if shell is interactive
        """
        return False


class CloudSetupLogger(LogsLogger):
    def __init__(
        self,
        log_output: bool = True,
        t0: float = _process_start_time,
        spinner_manager: Optional[Halo] = None,
    ):
        self.cloud_resource_errors: List[CloudAnalyticsEventCloudProviderError] = []
        super().__init__(log_output, t0, spinner_manager)

    def log_resource_exception(
        self, resource: CloudAnalyticsEventCloudResource, exc: Exception
    ):
        """
        Record the error in the logger if the exception is an unhandled exception
        """
        if resource in (
            CloudAnalyticsEventCloudResource.GCP_DEPLOYMENT,
            CloudAnalyticsEventCloudResource.GCP_FILESTORE,
            CloudAnalyticsEventCloudResource.GCP_FIREWALL_POLICY,
            CloudAnalyticsEventCloudResource.GCP_PROJECT,
            CloudAnalyticsEventCloudResource.GCP_SERVICE_ACCOUNT,
            CloudAnalyticsEventCloudResource.GCP_STORAGE_BUCKET,
            CloudAnalyticsEventCloudResource.GCP_SUBNET,
            CloudAnalyticsEventCloudResource.GCP_VPC,
            CloudAnalyticsEventCloudResource.GCP_WORKLOAD_IDENTITY_PROVIDER,
        ):
            # exceptions from GCP
            GoogleAPICallError, HttpError = try_import_gcp_exceptions()
            if isinstance(exc, GoogleAPICallError):
                self.log_resource_error(
                    resource, exc.reason, exc.code, "GoogleAPICallError",
                )
                return
            elif isinstance(exc, HttpError):
                reason = None
                with contextlib.suppress(Exception):
                    for error_detail in exc.error_details:
                        if error_detail.get("reason"):
                            reason = str(error_detail["reason"])
                            break
                self.log_resource_error(resource, reason, exc.status_code, "HttpError")
                return
        elif isinstance(exc, ClientError):
            # boto3 client error
            self.log_resource_error(
                resource,
                exc.response.get("Error", {}).get("Code"),
                exc.response.get("ResponseMetadata", {}).get("HTTPStatusCode"),
                "ClientError",
            )
            return
        else:
            unhandled_exception = f"UnknownExceptionType_{exc.__class__.__name__}"
            self.log_resource_error(
                resource, None, None, unhandled_exception,
            )
            return

    def log_resource_error(
        self,
        cloud_resource: CloudAnalyticsEventCloudResource,
        error_reason: Optional[str] = None,
        status_code: Optional[int] = None,
        unhandled_exception: Optional[str] = None,
    ):
        """
        Append the error in the logger. If the error is already logged, it will be deduplicated.
        """
        formatted_error_str = error_reason if error_reason else "unknown"
        if status_code:
            formatted_error_str = f"{formatted_error_str},{status_code}"
        if unhandled_exception:
            formatted_error_str = f"{formatted_error_str},{unhandled_exception}"
        for error in self.cloud_resource_errors:
            if (
                error.cloud_resource == cloud_resource
                and error.error_code == formatted_error_str
            ):
                # deduplicated errors
                return
        self.cloud_resource_errors.append(
            CloudAnalyticsEventCloudProviderError(
                cloud_resource=cloud_resource, error_code=formatted_error_str
            )
        )

    def get_cloud_provider_errors(self) -> List[CloudAnalyticsEventCloudProviderError]:
        return self.cloud_resource_errors

    def clear_cloud_provider_errors(self):
        self.cloud_resource_errors = []



from typing import Optional

from anyscale._private.sdk import sdk_command
from anyscale.compute_config._private.compute_config_sdk import PrivateComputeConfigSDK
from anyscale.compute_config.models import (
    ComputeConfigType,
    ComputeConfigVersion,
)


_COMPUTE_CONFIG_SDK_SINGLETON_KEY = "compute_config_sdk"

_CREATE_EXAMPLE = """
import anyscale
from anyscale.compute_config.models import ComputeConfig, HeadNodeConfig, MarketType, WorkerNodeGroupConfig

single_deployment_compute_config = ComputeConfig(
    head_node=HeadNodeConfig(
        instance_type="m5.8xlarge",
    ),
    worker_nodes=[
        WorkerNodeGroupConfig(
            instance_type="m5.8xlarge",
            min_nodes=5,
            max_nodes=5,
        ),
        WorkerNodeGroupConfig(
            instance_type="m5.4xlarge",
            min_nodes=1,
            max_nodes=10,
            market_type=MarketType.SPOT,
        ),
    ],
)
full_name: str = anyscale.compute_config.create(single_deployment_compute_config, name="my-single-deployment-compute-config")

multi_deployment_compute_config = MultiDeploymentComputeConfig(
    configs=[
        ComputeConfig(
            cloud_deployment="vm-aws-us-west-1",
            head_node=HeadNodeConfig(
                instance_type="m5.2xlarge",
            ),
            worker_nodes=[
                WorkerNodeGroupConfig(
                    instance_type="m5.4xlarge",
                    min_nodes=1,
                    max_nodes=10,
                ),
            ],
        ),
        ComputeConfig(
            cloud_deployment="vm-aws-us-west-2",
            head_node=HeadNodeConfig(
                instance_type="m5.2xlarge",
            ),
            worker_nodes=[
                WorkerNodeGroupConfig(
                    instance_type="m5.4xlarge",
                    min_nodes=1,
                    max_nodes=10,
                ),
            ],
        )
    ]
)
full_name: str = anyscale.compute_config.create(multi_deployment_compute_config, name="my-multi-deployment-compute-config")
"""

_CREATE_ARG_DOCSTRINGS = {
    "config": "The config options defining the compute config.",
    "name": "The name of the compute config. This should *not* include a version tag. If a name is not provided, one will be automatically generated.",
}


@sdk_command(
    _COMPUTE_CONFIG_SDK_SINGLETON_KEY,
    PrivateComputeConfigSDK,
    doc_py_example=_CREATE_EXAMPLE,
    arg_docstrings=_CREATE_ARG_DOCSTRINGS,
)
def create(
    config: ComputeConfigType,
    *,
    name: Optional[str],
    _private_sdk: Optional[PrivateComputeConfigSDK] = None,
) -> str:
    """Create a new version of a compute config.

    Returns the full name of the registered compute config, including the version.
    """
    full_name, _ = _private_sdk.create_compute_config(config, name=name)  # type: ignore
    return full_name


_GET_EXAMPLE = """
import anyscale
from anyscale.compute_config.models import ComputeConfig

compute_config: ComputeConfig = anyscale.compute_config.get("my-compute-config")
"""
_GET_ARG_DOCSTRINGS = {
    "name": "The name of the compute config. This can inclue an optional version tag, i.e., 'name:version'. If no version tag is provided, the latest version will be returned.",
    "include_archived": "Whether to consider archived compute configs (defaults to False).",
}


@sdk_command(
    _COMPUTE_CONFIG_SDK_SINGLETON_KEY,
    PrivateComputeConfigSDK,
    doc_py_example=_GET_EXAMPLE,
    arg_docstrings=_GET_ARG_DOCSTRINGS,
)
def get(
    name: Optional[str],
    *,
    include_archived: bool = False,
    _id: Optional[str] = None,
    _private_sdk: Optional[PrivateComputeConfigSDK] = None,
) -> ComputeConfigVersion:
    """Get the compute config with the specified name.

    The name can contain an optional version tag, i.e., 'name:version'.
    If no version is provided, the latest one will be returned.
    """
    # NOTE(edoakes): I want to avoid exposing fetching by ID in the public API,
    # but it's needed for parity with the existing CLI. Therefore I am adding it
    # as a hidden private API that can be used like: (`name="", _id=id`).
    return _private_sdk.get_compute_config(  # type: ignore
        name=name or None, id=_id, include_archived=include_archived
    )


_ARCHIVE_EXAMPLE = """
import anyscale

anyscale.compute_config.archive(name="my-compute-config")
"""

_ARCHIVE_ARG_DOCSTRINGS = {"name": "Name of the compute config."}


@sdk_command(
    _COMPUTE_CONFIG_SDK_SINGLETON_KEY,
    PrivateComputeConfigSDK,
    doc_py_example=_ARCHIVE_EXAMPLE,
    arg_docstrings=_ARCHIVE_ARG_DOCSTRINGS,
)
def archive(
    name: Optional[str],
    *,
    _id: Optional[str] = None,
    _private_sdk: Optional[PrivateComputeConfigSDK] = None,
):
    """Archive a compute config and all of its versions.

    The name can contain an optional version, e.g., 'name:version'.
    If no version is provided, the latest one will be archived.

    Once a compute config is archived, its name will no longer be usable in the organization.
    """
    # NOTE(edoakes): I want to avoid exposing fetching by ID in the public API,
    # but it's needed for parity with the existing CLI. Therefore I am adding it
    # as a hidden private API that can be used like: (`name="", _id=id`).
    return _private_sdk.archive_compute_config(name=name or None, id=_id)  # type: ignore



from typing import List, Optional

from anyscale._private.anyscale_client import AnyscaleClient
from anyscale._private.models.model_base import ResultIterator
from anyscale._private.sdk import sdk_command, sdk_docs
from anyscale._private.sdk.base_sdk import Timer
from anyscale.cli_logger import BlockLogger
from anyscale.client.openapi_client.models.job_queue_sort_directive import (
    JobQueueSortDirective,
)
from anyscale.client.openapi_client.models.session_state import SessionState
from anyscale.job_queue._private.job_queue_sdk import PrivateJobQueueSDK
from anyscale.job_queue.commands import (
    _JOB_QUEUE_SDK_SINGLETON_KEY,
    _LIST_ARG_DOCSTRINGS,
    _LIST_EXAMPLE,
    _STATUS_ARG_DOCSTRINGS,
    _STATUS_EXAMPLE,
    _UPDATE_ARG_DOCSTRINGS,
    _UPDATE_EXAMPLE,
    list,
    status,
    update,
)
from anyscale.job_queue.models import JobQueueSortField, JobQueueState, JobQueueStatus


class JobQueueSDK:
    """Public SDK for interacting with Anyscale Job Queues."""

    def __init__(
        self,
        *,
        client: Optional[AnyscaleClient] = None,
        logger: Optional[BlockLogger] = None,
        timer: Optional[Timer] = None,
    ):
        self._private_sdk = PrivateJobQueueSDK(
            client=client, logger=logger, timer=timer
        )

    @sdk_docs(doc_py_example=_LIST_EXAMPLE, arg_docstrings=_LIST_ARG_DOCSTRINGS)
    def list(  # noqa: F811
        self,
        *,
        job_queue_id: Optional[str] = None,
        name: Optional[str] = None,
        creator_id: Optional[str] = None,
        cloud: Optional[str] = None,
        project: Optional[str] = None,
        cluster_status: Optional[SessionState] = None,
        page_size: Optional[int] = None,
        max_items: Optional[int] = None,
        sorting_directives: Optional[List[JobQueueSortDirective]] = None,
    ) -> ResultIterator[JobQueueStatus]:
        """List job queues or fetch a single job queue by ID."""
        return self._private_sdk.list(
            job_queue_id=job_queue_id,
            name=name,
            creator_id=creator_id,
            cloud=cloud,
            project=project,
            cluster_status=cluster_status,
            page_size=page_size,
            max_items=max_items,
            sorting_directives=sorting_directives,
        )

    @sdk_docs(doc_py_example=_STATUS_EXAMPLE, arg_docstrings=_STATUS_ARG_DOCSTRINGS)
    def status(self, job_queue_id: str) -> JobQueueStatus:  # noqa: F811
        """Get the status and details for a specific job queue."""
        return self._private_sdk.status(job_queue_id=job_queue_id)

    @sdk_docs(doc_py_example=_UPDATE_EXAMPLE, arg_docstrings=_UPDATE_ARG_DOCSTRINGS)
    def update(  # noqa: F811
        self,
        *,
        job_queue_id: Optional[str] = None,
        job_queue_name: Optional[str] = None,
        max_concurrency: Optional[int] = None,
        idle_timeout_s: Optional[int] = None,
    ) -> JobQueueStatus:
        """Update a job queue."""
        return self._private_sdk.update(
            job_queue_id=job_queue_id,
            job_queue_name=job_queue_name,
            max_concurrency=max_concurrency,
            idle_timeout_s=idle_timeout_s,
        )



# coding: utf-8

"""
    Managed Ray API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 0.1.0
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from openapi_client.configuration import Configuration


class DatasetMetrics(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'id': 'str',
        'name': 'str',
        'job_id': 'str',
        'session_name': 'str',
        'state': 'DatasetState',
        'progress': 'int',
        'total': 'int',
        'start_time': 'float',
        'end_time': 'float',
        'operator_metrics': 'list[OperatorMetrics]',
        'metrics': 'dict(str, Metric)'
    }

    attribute_map = {
        'id': 'id',
        'name': 'name',
        'job_id': 'job_id',
        'session_name': 'session_name',
        'state': 'state',
        'progress': 'progress',
        'total': 'total',
        'start_time': 'start_time',
        'end_time': 'end_time',
        'operator_metrics': 'operator_metrics',
        'metrics': 'metrics'
    }

    def __init__(self, id=None, name=None, job_id=None, session_name=None, state=None, progress=None, total=None, start_time=None, end_time=None, operator_metrics=None, metrics=None, local_vars_configuration=None):  # noqa: E501
        """DatasetMetrics - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._id = None
        self._name = None
        self._job_id = None
        self._session_name = None
        self._state = None
        self._progress = None
        self._total = None
        self._start_time = None
        self._end_time = None
        self._operator_metrics = None
        self._metrics = None
        self.discriminator = None

        self.id = id
        self.name = name
        self.job_id = job_id
        self.session_name = session_name
        self.state = state
        self.progress = progress
        if total is not None:
            self.total = total
        self.start_time = start_time
        if end_time is not None:
            self.end_time = end_time
        self.operator_metrics = operator_metrics
        self.metrics = metrics

    @property
    def id(self):
        """Gets the id of this DatasetMetrics.  # noqa: E501


        :return: The id of this DatasetMetrics.  # noqa: E501
        :rtype: str
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this DatasetMetrics.


        :param id: The id of this DatasetMetrics.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and id is None:  # noqa: E501
            raise ValueError("Invalid value for `id`, must not be `None`")  # noqa: E501

        self._id = id

    @property
    def name(self):
        """Gets the name of this DatasetMetrics.  # noqa: E501


        :return: The name of this DatasetMetrics.  # noqa: E501
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this DatasetMetrics.


        :param name: The name of this DatasetMetrics.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and name is None:  # noqa: E501
            raise ValueError("Invalid value for `name`, must not be `None`")  # noqa: E501

        self._name = name

    @property
    def job_id(self):
        """Gets the job_id of this DatasetMetrics.  # noqa: E501


        :return: The job_id of this DatasetMetrics.  # noqa: E501
        :rtype: str
        """
        return self._job_id

    @job_id.setter
    def job_id(self, job_id):
        """Sets the job_id of this DatasetMetrics.


        :param job_id: The job_id of this DatasetMetrics.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and job_id is None:  # noqa: E501
            raise ValueError("Invalid value for `job_id`, must not be `None`")  # noqa: E501

        self._job_id = job_id

    @property
    def session_name(self):
        """Gets the session_name of this DatasetMetrics.  # noqa: E501


        :return: The session_name of this DatasetMetrics.  # noqa: E501
        :rtype: str
        """
        return self._session_name

    @session_name.setter
    def session_name(self, session_name):
        """Sets the session_name of this DatasetMetrics.


        :param session_name: The session_name of this DatasetMetrics.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and session_name is None:  # noqa: E501
            raise ValueError("Invalid value for `session_name`, must not be `None`")  # noqa: E501

        self._session_name = session_name

    @property
    def state(self):
        """Gets the state of this DatasetMetrics.  # noqa: E501


        :return: The state of this DatasetMetrics.  # noqa: E501
        :rtype: DatasetState
        """
        return self._state

    @state.setter
    def state(self, state):
        """Sets the state of this DatasetMetrics.


        :param state: The state of this DatasetMetrics.  # noqa: E501
        :type: DatasetState
        """
        if self.local_vars_configuration.client_side_validation and state is None:  # noqa: E501
            raise ValueError("Invalid value for `state`, must not be `None`")  # noqa: E501

        self._state = state

    @property
    def progress(self):
        """Gets the progress of this DatasetMetrics.  # noqa: E501


        :return: The progress of this DatasetMetrics.  # noqa: E501
        :rtype: int
        """
        return self._progress

    @progress.setter
    def progress(self, progress):
        """Sets the progress of this DatasetMetrics.


        :param progress: The progress of this DatasetMetrics.  # noqa: E501
        :type: int
        """
        if self.local_vars_configuration.client_side_validation and progress is None:  # noqa: E501
            raise ValueError("Invalid value for `progress`, must not be `None`")  # noqa: E501

        self._progress = progress

    @property
    def total(self):
        """Gets the total of this DatasetMetrics.  # noqa: E501


        :return: The total of this DatasetMetrics.  # noqa: E501
        :rtype: int
        """
        return self._total

    @total.setter
    def total(self, total):
        """Sets the total of this DatasetMetrics.


        :param total: The total of this DatasetMetrics.  # noqa: E501
        :type: int
        """

        self._total = total

    @property
    def start_time(self):
        """Gets the start_time of this DatasetMetrics.  # noqa: E501


        :return: The start_time of this DatasetMetrics.  # noqa: E501
        :rtype: float
        """
        return self._start_time

    @start_time.setter
    def start_time(self, start_time):
        """Sets the start_time of this DatasetMetrics.


        :param start_time: The start_time of this DatasetMetrics.  # noqa: E501
        :type: float
        """
        if self.local_vars_configuration.client_side_validation and start_time is None:  # noqa: E501
            raise ValueError("Invalid value for `start_time`, must not be `None`")  # noqa: E501

        self._start_time = start_time

    @property
    def end_time(self):
        """Gets the end_time of this DatasetMetrics.  # noqa: E501


        :return: The end_time of this DatasetMetrics.  # noqa: E501
        :rtype: float
        """
        return self._end_time

    @end_time.setter
    def end_time(self, end_time):
        """Sets the end_time of this DatasetMetrics.


        :param end_time: The end_time of this DatasetMetrics.  # noqa: E501
        :type: float
        """

        self._end_time = end_time

    @property
    def operator_metrics(self):
        """Gets the operator_metrics of this DatasetMetrics.  # noqa: E501


        :return: The operator_metrics of this DatasetMetrics.  # noqa: E501
        :rtype: list[OperatorMetrics]
        """
        return self._operator_metrics

    @operator_metrics.setter
    def operator_metrics(self, operator_metrics):
        """Sets the operator_metrics of this DatasetMetrics.


        :param operator_metrics: The operator_metrics of this DatasetMetrics.  # noqa: E501
        :type: list[OperatorMetrics]
        """
        if self.local_vars_configuration.client_side_validation and operator_metrics is None:  # noqa: E501
            raise ValueError("Invalid value for `operator_metrics`, must not be `None`")  # noqa: E501

        self._operator_metrics = operator_metrics

    @property
    def metrics(self):
        """Gets the metrics of this DatasetMetrics.  # noqa: E501


        :return: The metrics of this DatasetMetrics.  # noqa: E501
        :rtype: dict(str, Metric)
        """
        return self._metrics

    @metrics.setter
    def metrics(self, metrics):
        """Sets the metrics of this DatasetMetrics.


        :param metrics: The metrics of this DatasetMetrics.  # noqa: E501
        :type: dict(str, Metric)
        """
        if self.local_vars_configuration.client_side_validation and metrics is None:  # noqa: E501
            raise ValueError("Invalid value for `metrics`, must not be `None`")  # noqa: E501

        self._metrics = metrics

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, DatasetMetrics):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, DatasetMetrics):
            return True

        return self.to_dict() != other.to_dict()



# coding: utf-8

"""
    Managed Ray API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 0.1.0
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from openapi_client.configuration import Configuration


class SummarizeMachinePoolResponse(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'partitions': 'list[PartitionInfo]',
        'running_workloads': 'int',
        'queued_workloads': 'int'
    }

    attribute_map = {
        'partitions': 'partitions',
        'running_workloads': 'running_workloads',
        'queued_workloads': 'queued_workloads'
    }

    def __init__(self, partitions=None, running_workloads=None, queued_workloads=None, local_vars_configuration=None):  # noqa: E501
        """SummarizeMachinePoolResponse - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._partitions = None
        self._running_workloads = None
        self._queued_workloads = None
        self.discriminator = None

        self.partitions = partitions
        self.running_workloads = running_workloads
        self.queued_workloads = queued_workloads

    @property
    def partitions(self):
        """Gets the partitions of this SummarizeMachinePoolResponse.  # noqa: E501

        Summary for each partition in the machine pool.  # noqa: E501

        :return: The partitions of this SummarizeMachinePoolResponse.  # noqa: E501
        :rtype: list[PartitionInfo]
        """
        return self._partitions

    @partitions.setter
    def partitions(self, partitions):
        """Sets the partitions of this SummarizeMachinePoolResponse.

        Summary for each partition in the machine pool.  # noqa: E501

        :param partitions: The partitions of this SummarizeMachinePoolResponse.  # noqa: E501
        :type: list[PartitionInfo]
        """
        if self.local_vars_configuration.client_side_validation and partitions is None:  # noqa: E501
            raise ValueError("Invalid value for `partitions`, must not be `None`")  # noqa: E501

        self._partitions = partitions

    @property
    def running_workloads(self):
        """Gets the running_workloads of this SummarizeMachinePoolResponse.  # noqa: E501

        The number of running workloads.  # noqa: E501

        :return: The running_workloads of this SummarizeMachinePoolResponse.  # noqa: E501
        :rtype: int
        """
        return self._running_workloads

    @running_workloads.setter
    def running_workloads(self, running_workloads):
        """Sets the running_workloads of this SummarizeMachinePoolResponse.

        The number of running workloads.  # noqa: E501

        :param running_workloads: The running_workloads of this SummarizeMachinePoolResponse.  # noqa: E501
        :type: int
        """
        if self.local_vars_configuration.client_side_validation and running_workloads is None:  # noqa: E501
            raise ValueError("Invalid value for `running_workloads`, must not be `None`")  # noqa: E501

        self._running_workloads = running_workloads

    @property
    def queued_workloads(self):
        """Gets the queued_workloads of this SummarizeMachinePoolResponse.  # noqa: E501

        The number of queued workloads.  # noqa: E501

        :return: The queued_workloads of this SummarizeMachinePoolResponse.  # noqa: E501
        :rtype: int
        """
        return self._queued_workloads

    @queued_workloads.setter
    def queued_workloads(self, queued_workloads):
        """Sets the queued_workloads of this SummarizeMachinePoolResponse.

        The number of queued workloads.  # noqa: E501

        :param queued_workloads: The queued_workloads of this SummarizeMachinePoolResponse.  # noqa: E501
        :type: int
        """
        if self.local_vars_configuration.client_side_validation and queued_workloads is None:  # noqa: E501
            raise ValueError("Invalid value for `queued_workloads`, must not be `None`")  # noqa: E501

        self._queued_workloads = queued_workloads

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, SummarizeMachinePoolResponse):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, SummarizeMachinePoolResponse):
            return True

        return self.to_dict() != other.to_dict()



# coding: utf-8

"""
    Managed Ray API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 0.1.0
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from openapi_client.configuration import Configuration


class JobDetails(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'job_id': 'str',
        'submission_id': 'str',
        'runtime_env': 'object',
        'entrypoint': 'str'
    }

    attribute_map = {
        'job_id': 'job_id',
        'submission_id': 'submission_id',
        'runtime_env': 'runtime_env',
        'entrypoint': 'entrypoint'
    }

    def __init__(self, job_id=None, submission_id=None, runtime_env=None, entrypoint=None, local_vars_configuration=None):  # noqa: E501
        """JobDetails - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._job_id = None
        self._submission_id = None
        self._runtime_env = None
        self._entrypoint = None
        self.discriminator = None

        if job_id is not None:
            self.job_id = job_id
        if submission_id is not None:
            self.submission_id = submission_id
        if runtime_env is not None:
            self.runtime_env = runtime_env
        self.entrypoint = entrypoint

    @property
    def job_id(self):
        """Gets the job_id of this JobDetails.  # noqa: E501


        :return: The job_id of this JobDetails.  # noqa: E501
        :rtype: str
        """
        return self._job_id

    @job_id.setter
    def job_id(self, job_id):
        """Sets the job_id of this JobDetails.


        :param job_id: The job_id of this JobDetails.  # noqa: E501
        :type: str
        """

        self._job_id = job_id

    @property
    def submission_id(self):
        """Gets the submission_id of this JobDetails.  # noqa: E501


        :return: The submission_id of this JobDetails.  # noqa: E501
        :rtype: str
        """
        return self._submission_id

    @submission_id.setter
    def submission_id(self, submission_id):
        """Sets the submission_id of this JobDetails.


        :param submission_id: The submission_id of this JobDetails.  # noqa: E501
        :type: str
        """

        self._submission_id = submission_id

    @property
    def runtime_env(self):
        """Gets the runtime_env of this JobDetails.  # noqa: E501


        :return: The runtime_env of this JobDetails.  # noqa: E501
        :rtype: object
        """
        return self._runtime_env

    @runtime_env.setter
    def runtime_env(self, runtime_env):
        """Sets the runtime_env of this JobDetails.


        :param runtime_env: The runtime_env of this JobDetails.  # noqa: E501
        :type: object
        """

        self._runtime_env = runtime_env

    @property
    def entrypoint(self):
        """Gets the entrypoint of this JobDetails.  # noqa: E501


        :return: The entrypoint of this JobDetails.  # noqa: E501
        :rtype: str
        """
        return self._entrypoint

    @entrypoint.setter
    def entrypoint(self, entrypoint):
        """Sets the entrypoint of this JobDetails.


        :param entrypoint: The entrypoint of this JobDetails.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and entrypoint is None:  # noqa: E501
            raise ValueError("Invalid value for `entrypoint`, must not be `None`")  # noqa: E501

        self._entrypoint = entrypoint

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, JobDetails):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, JobDetails):
            return True

        return self.to_dict() != other.to_dict()



import pathlib
from typing import Optional

from rich.console import Console
import yaml

from anyscale.cli_logger import BlockLogger
from anyscale.client.openapi_client.models import (
    AttachMachinePoolToCloudRequest,
    CreateMachinePoolRequest,
    CreateMachinePoolResponse,
    DeleteMachinePoolRequest,
    DescribeMachinePoolRequest,
    DescribeMachinePoolResponse,
    DetachMachinePoolFromCloudRequest,
    ListMachinePoolsResponse,
    UpdateMachinePoolRequest,
)
from anyscale.cloud_utils import get_cloud_id_and_name
from anyscale.controllers.base_controller import BaseController


class MachinePoolController(BaseController):
    def __init__(
        self, log: Optional[BlockLogger] = None, initialize_auth_api_client: bool = True
    ):
        if log is None:
            log = BlockLogger()

        super().__init__(initialize_auth_api_client=initialize_auth_api_client)
        self.log = log
        self.console = Console()

    def create_machine_pool(
        self,
        machine_pool_name: str,
        enable_rootless_dataplane_config: Optional[bool] = False,
    ) -> CreateMachinePoolResponse:
        response: CreateMachinePoolResponse = self.api_client.create_machine_pool_api_v2_machine_pools_create_post(
            CreateMachinePoolRequest(
                machine_pool_name=machine_pool_name,
                enable_rootless_dataplane_config=enable_rootless_dataplane_config,
            )
        ).result
        return response

    def delete_machine_pool(
        self, machine_pool_name: str,
    ):
        self.api_client.delete_machine_pool_api_v2_machine_pools_delete_post(
            DeleteMachinePoolRequest(machine_pool_name=machine_pool_name)
        )

    def update_machine_pool(
        self, machine_pool_name: str, spec_file: str,
    ):
        path = pathlib.Path(spec_file)
        if not path.exists():
            raise FileNotFoundError(f"File {spec_file} does not exist.")

        if not path.is_file():
            raise ValueError(f"File {spec_file} is not a file.")

        spec = yaml.safe_load(path.read_text())

        self.api_client.update_machine_pool_api_v2_machine_pools_update_post(
            UpdateMachinePoolRequest(machine_pool_name=machine_pool_name, spec=spec,)
        )

    def describe_machine_pool(
        self, machine_pool_name: str,
    ) -> DescribeMachinePoolResponse:
        return self.api_client.describe_machine_pool_api_v2_machine_pools_describe_post(
            describe_machine_pool_request=DescribeMachinePoolRequest(
                machine_pool_name=machine_pool_name,
            )
        ).result

    def list_machine_pools(self,) -> ListMachinePoolsResponse:
        response = self.api_client.list_machine_pools_api_v2_machine_pools_get().result
        return response

    def attach_machine_pool_to_cloud(self, machine_pool_name: str, cloud: str):
        cloud_id, _ = get_cloud_id_and_name(self.api_client, cloud_name=cloud)
        self.api_client.attach_machine_pool_to_cloud_api_v2_machine_pools_attach_post(
            AttachMachinePoolToCloudRequest(
                machine_pool_name=machine_pool_name, cloud_id=cloud_id
            )
        )

    def detach_machine_pool_from_cloud(self, machine_pool_name: str, cloud: str):
        cloud_id, _ = get_cloud_id_and_name(self.api_client, cloud_name=cloud)
        self.api_client.detach_machine_pool_from_cloud_api_v2_machine_pools_detach_post(
            DetachMachinePoolFromCloudRequest(
                machine_pool_name=machine_pool_name, cloud_id=cloud_id
            )
        )



# coding: utf-8

"""
    Managed Ray API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 0.1.0
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from openapi_client.configuration import Configuration


class MiniCloud(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'id': 'str',
        'name': 'str',
        'provider': 'CloudProviders',
        'is_k8s': 'bool',
        'is_aioa': 'bool',
        'cluster_management_stack_version': 'ClusterManagementStackVersions',
        'is_private_cloud': 'bool',
        'customer_aggregated_logs_enabled': 'bool'
    }

    attribute_map = {
        'id': 'id',
        'name': 'name',
        'provider': 'provider',
        'is_k8s': 'is_k8s',
        'is_aioa': 'is_aioa',
        'cluster_management_stack_version': 'cluster_management_stack_version',
        'is_private_cloud': 'is_private_cloud',
        'customer_aggregated_logs_enabled': 'customer_aggregated_logs_enabled'
    }

    def __init__(self, id=None, name=None, provider=None, is_k8s=False, is_aioa=False, cluster_management_stack_version=None, is_private_cloud=False, customer_aggregated_logs_enabled=False, local_vars_configuration=None):  # noqa: E501
        """MiniCloud - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._id = None
        self._name = None
        self._provider = None
        self._is_k8s = None
        self._is_aioa = None
        self._cluster_management_stack_version = None
        self._is_private_cloud = None
        self._customer_aggregated_logs_enabled = None
        self.discriminator = None

        self.id = id
        self.name = name
        self.provider = provider
        if is_k8s is not None:
            self.is_k8s = is_k8s
        if is_aioa is not None:
            self.is_aioa = is_aioa
        if cluster_management_stack_version is not None:
            self.cluster_management_stack_version = cluster_management_stack_version
        if is_private_cloud is not None:
            self.is_private_cloud = is_private_cloud
        if customer_aggregated_logs_enabled is not None:
            self.customer_aggregated_logs_enabled = customer_aggregated_logs_enabled

    @property
    def id(self):
        """Gets the id of this MiniCloud.  # noqa: E501

        The id of the cloud  # noqa: E501

        :return: The id of this MiniCloud.  # noqa: E501
        :rtype: str
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this MiniCloud.

        The id of the cloud  # noqa: E501

        :param id: The id of this MiniCloud.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and id is None:  # noqa: E501
            raise ValueError("Invalid value for `id`, must not be `None`")  # noqa: E501

        self._id = id

    @property
    def name(self):
        """Gets the name of this MiniCloud.  # noqa: E501

        The name of this cloud  # noqa: E501

        :return: The name of this MiniCloud.  # noqa: E501
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this MiniCloud.

        The name of this cloud  # noqa: E501

        :param name: The name of this MiniCloud.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and name is None:  # noqa: E501
            raise ValueError("Invalid value for `name`, must not be `None`")  # noqa: E501

        self._name = name

    @property
    def provider(self):
        """Gets the provider of this MiniCloud.  # noqa: E501

        The cloud provider for this cloud  # noqa: E501

        :return: The provider of this MiniCloud.  # noqa: E501
        :rtype: CloudProviders
        """
        return self._provider

    @provider.setter
    def provider(self, provider):
        """Sets the provider of this MiniCloud.

        The cloud provider for this cloud  # noqa: E501

        :param provider: The provider of this MiniCloud.  # noqa: E501
        :type: CloudProviders
        """
        if self.local_vars_configuration.client_side_validation and provider is None:  # noqa: E501
            raise ValueError("Invalid value for `provider`, must not be `None`")  # noqa: E501

        self._provider = provider

    @property
    def is_k8s(self):
        """Gets the is_k8s of this MiniCloud.  # noqa: E501

        Whether this cloud is managed via K8s  # noqa: E501

        :return: The is_k8s of this MiniCloud.  # noqa: E501
        :rtype: bool
        """
        return self._is_k8s

    @is_k8s.setter
    def is_k8s(self, is_k8s):
        """Sets the is_k8s of this MiniCloud.

        Whether this cloud is managed via K8s  # noqa: E501

        :param is_k8s: The is_k8s of this MiniCloud.  # noqa: E501
        :type: bool
        """

        self._is_k8s = is_k8s

    @property
    def is_aioa(self):
        """Gets the is_aioa of this MiniCloud.  # noqa: E501

        Whether this cloud is an AIOA cloud.  # noqa: E501

        :return: The is_aioa of this MiniCloud.  # noqa: E501
        :rtype: bool
        """
        return self._is_aioa

    @is_aioa.setter
    def is_aioa(self, is_aioa):
        """Sets the is_aioa of this MiniCloud.

        Whether this cloud is an AIOA cloud.  # noqa: E501

        :param is_aioa: The is_aioa of this MiniCloud.  # noqa: E501
        :type: bool
        """

        self._is_aioa = is_aioa

    @property
    def cluster_management_stack_version(self):
        """Gets the cluster_management_stack_version of this MiniCloud.  # noqa: E501

        The cluster management stack version of the cloud.  # noqa: E501

        :return: The cluster_management_stack_version of this MiniCloud.  # noqa: E501
        :rtype: ClusterManagementStackVersions
        """
        return self._cluster_management_stack_version

    @cluster_management_stack_version.setter
    def cluster_management_stack_version(self, cluster_management_stack_version):
        """Sets the cluster_management_stack_version of this MiniCloud.

        The cluster management stack version of the cloud.  # noqa: E501

        :param cluster_management_stack_version: The cluster_management_stack_version of this MiniCloud.  # noqa: E501
        :type: ClusterManagementStackVersions
        """

        self._cluster_management_stack_version = cluster_management_stack_version

    @property
    def is_private_cloud(self):
        """Gets the is_private_cloud of this MiniCloud.  # noqa: E501

        Whether this cloud is a private cloud. This is only possible if clouds is a V2 cloud.  # noqa: E501

        :return: The is_private_cloud of this MiniCloud.  # noqa: E501
        :rtype: bool
        """
        return self._is_private_cloud

    @is_private_cloud.setter
    def is_private_cloud(self, is_private_cloud):
        """Sets the is_private_cloud of this MiniCloud.

        Whether this cloud is a private cloud. This is only possible if clouds is a V2 cloud.  # noqa: E501

        :param is_private_cloud: The is_private_cloud of this MiniCloud.  # noqa: E501
        :type: bool
        """

        self._is_private_cloud = is_private_cloud

    @property
    def customer_aggregated_logs_enabled(self):
        """Gets the customer_aggregated_logs_enabled of this MiniCloud.  # noqa: E501

        Whether this cloud has customer aggregated logs enabled.  # noqa: E501

        :return: The customer_aggregated_logs_enabled of this MiniCloud.  # noqa: E501
        :rtype: bool
        """
        return self._customer_aggregated_logs_enabled

    @customer_aggregated_logs_enabled.setter
    def customer_aggregated_logs_enabled(self, customer_aggregated_logs_enabled):
        """Sets the customer_aggregated_logs_enabled of this MiniCloud.

        Whether this cloud has customer aggregated logs enabled.  # noqa: E501

        :param customer_aggregated_logs_enabled: The customer_aggregated_logs_enabled of this MiniCloud.  # noqa: E501
        :type: bool
        """

        self._customer_aggregated_logs_enabled = customer_aggregated_logs_enabled

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, MiniCloud):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, MiniCloud):
            return True

        return self.to_dict() != other.to_dict()



from dataclasses import dataclass, field
from typing import Dict, List, Optional, Union

from anyscale._private.models import ImageURI, ModelBase
from anyscale._private.models.model_base import ModelEnum
from anyscale.compute_config.models import (
    compute_config_type_from_dict,
    ComputeConfig,
    ComputeConfigType,
    MultiDeploymentComputeConfig,
)


class WorkspaceState(ModelEnum):

    STARTING = "STARTING"
    UPDATING = "UPDATING"
    RUNNING = "RUNNING"
    TERMINATING = "TERMINATING"
    TERMINATED = "TERMINATED"
    ERRORED = "ERRORED"
    UNKNOWN = "UNKNOWN"

    __docstrings__ = {
        "STARTING": "The workspace is starting up.",
        "UPDATING": "The workspace is updating.",
        "RUNNING": "The workspace is running.",
        "TERMINATING": "The workspace is terminating.",
        "TERMINATED": "The workspace is terminated.",
        "ERRORED": "The workspace is in an error state.",
        "UNKNOWN": "The workspace state",
    }


@dataclass(frozen=True)
class UpdateWorkspaceConfig(ModelBase):
    """Configuration options for updating a workspace."""

    __doc_py_example__ = """\
from anyscale.workspace.models import UpdateWorkspaceConfig

config = UpdateWorkspaceConfig(
    name="new-workpsace-name",
    idle_termination_minutes=220,
    env_vars={"key": "value"},
    requirements="/tmp/requirements.txt",
)
"""

    __doc_yaml_example__ = """\
name: new-workspace-name
idle_termination_minutes: 220
env_vars:
    key: value
requirements: /tmp/requirements.txt
"""

    name: Optional[str] = field(
        default=None, metadata={"docstring": "The name of the workspace"}
    )

    def _validate_name(self, name: Optional[str]):
        if name is not None and not isinstance(name, str):
            raise TypeError("'name' must be a string.")

    image_uri: Optional[str] = field(
        default=None,
        metadata={
            "docstring": "URI of an existing image. Exclusive with `containerfile`."
        },
    )

    def _validate_image_uri(self, image_uri: Optional[str]):
        if image_uri is not None and not isinstance(image_uri, str):
            raise TypeError(f"'image_uri' must be an str but it is {type(image_uri)}.")
        if image_uri is not None:
            ImageURI.from_str(image_uri)

    containerfile: Optional[str] = field(
        default=None,
        repr=False,
        metadata={
            "docstring": "The file path to a containerfile that will be built into an image before running the workload. Exclusive with `image_uri`."
        },
    )

    def _validate_containerfile(self, containerfile: Optional[str]):
        if containerfile is not None and not isinstance(containerfile, str):
            raise TypeError("'containerfile' must be a string.")

    compute_config: Union[ComputeConfigType, Dict, str, None] = field(
        default=None,
        metadata={
            "docstring": "The name of an existing registered compute config (including version) or an inlined ComputeConfig object."
        },
    )

    def _validate_compute_config(
        self, compute_config: Union[ComputeConfigType, Dict, str, None]
    ) -> Union[None, str, ComputeConfigType]:
        if compute_config is None or isinstance(compute_config, str):
            return compute_config

        if isinstance(compute_config, dict):
            compute_config = compute_config_type_from_dict(compute_config)
        if not isinstance(
            compute_config, (ComputeConfig, MultiDeploymentComputeConfig)
        ):
            raise TypeError(
                "'compute_config' must be a string, ComputeConfig, MultiDeploymentComputeConfig, or corresponding dict"
            )

        return compute_config

    idle_termination_minutes: Optional[int] = field(
        default=None,
        metadata={
            "docstring": "the time in minutes after which the workspace is terminated when idle."
        },
    )

    def _validate_idle_termination_minutes(
        self, idle_termination_minutes: Optional[int]
    ):
        if idle_termination_minutes is not None and not isinstance(
            idle_termination_minutes, int
        ):
            raise ValueError("'idle_termination_minutes' must be an int")

    requirements: Optional[Union[str, List[str]]] = field(
        default=None,
        repr=False,
        metadata={
            "docstring": "A list of pip requirements or a path to a `requirements.txt` file for the workload."
        },
    )

    def _validate_requirements(self, requirements: Optional[Union[str, List[str]]]):
        if requirements is None or isinstance(requirements, str):
            return

        if not isinstance(requirements, list) or not all(
            isinstance(r, str) for r in requirements
        ):
            raise TypeError(
                "'requirements' must be a string (file path) or list of strings."
            )

    env_vars: Optional[Dict[str, str]] = field(
        default=None,
        repr=True,
        metadata={
            "docstring": "A dictionary of environment variables that will be set for the workload."
        },
    )

    def _validate_env_vars(self, env_vars: Optional[Dict[str, str]]):
        if env_vars is not None and (
            not isinstance(env_vars, dict)
            or not all(
                isinstance(k, str) and isinstance(v, str) for k, v in env_vars.items()
            )
        ):
            raise TypeError("'env_vars' must be a Dict[str, str].")

    registry_login_secret: Optional[str] = field(
        default=None,
        repr=False,
        metadata={
            "docstring": "A name or identifier of the secret containing credentials to authenticate to the docker registry hosting the image. "
            "This can only be used when 'image_uri' is specified and the image is not hosted on Anyscale."
        },
    )

    def _validate_registry_login_secret(self, registry_login_secret: Optional[str]):
        if registry_login_secret is not None and not isinstance(
            registry_login_secret, str
        ):
            raise TypeError("'registry_login_secret' must be a string.")

    ray_version: Optional[str] = field(
        default=None,
        metadata={
            "docstring": "The Ray version (X.Y.Z) specified for this image specified by either an image URI or a containerfile. If you don't specify a Ray version, Anyscale defaults to the latest Ray version available at the time of the Anyscale CLI/SDK release."
        },
    )

    def _validate_ray_version(self, ray_version: Optional[str]):
        if ray_version is not None and not isinstance(ray_version, str):
            raise TypeError("'ray_version' must be a string.")
        if ray_version:
            vs = ray_version.split(".")
            if len(vs) != 3:
                raise ValueError(
                    f"Invalid Ray version format: {ray_version}. Must be in the format 'X.Y.Z'."
                )


@dataclass(frozen=True)
class WorkspaceConfig(UpdateWorkspaceConfig):
    """Configuration options for a workspace."""

    __doc_py_example__ = """\
from anyscale.workspace.models import WorkspaceConfig

config = WorkspaceConfig(
    name="my-workpsace",
    idle_termination_minutes=220,
    env_vars={"key": "value"},
    requirements="/tmp/requirements.txt",
)
"""

    __doc_yaml_example__ = """\
name: my-workspace
idle_termination_minutes: 220
env_vars:
    key: value
requirements: /tmp/requirements.txt
"""

    project: Optional[str] = field(
        default=None,
        metadata={
            "docstring": "The project for the workload. If not provided, the default project for the cloud will be used."
        },
    )

    def _validate_project(self, project: Optional[str]):
        if project is not None and not isinstance(project, str):
            raise TypeError("'project' must be a string.")

    cloud: Optional[str] = field(
        default=None,
        metadata={
            "docstring": "The Anyscale Cloud to run this workload on. If not provided, the organization default will be used."
        },
    )

    def _validate_cloud(self, cloud: Optional[str]):
        if cloud is not None and not isinstance(cloud, str):
            raise TypeError("'cloud' must be a string.")


@dataclass(frozen=True)
class Workspace(ModelBase):
    __doc_py_example__ = """\
import anyscale
from anyscale.workspace.models import Workspace

workspace: Workspace = anyscale.workspace.get(name="workspace-name")
"""

    __doc_cli_example__ = """\
$ anyscale workspace get -n my-workspace
id: expwrk_k13avulibzkx2tekkirvrhc9s4
name: my-workspace
state: TERMINATED
config:
  name: my-workspace
  project: default
  cloud: my-cloud
  image_uri: anyscale/ray:2.37.0-slim-py312-cu123
  compute_config:
    cloud: my-cloud
    head_node:
      instance_type: m5.2xlarge
    worker_nodes: []
    enable_cross_zone_scaling: false
  idle_termination_minutes: 120
"""

    id: str = field(metadata={"docstring": "The unique identifier of the workspace."})

    def _validate_id(self, id: str):  # noqa: A002
        if not isinstance(id, str):
            raise ValueError("'id' must be a string.")

    name: str = field(metadata={"docstring": "The name of the workspace"})

    def _validate_name(self, name: str):
        if not isinstance(name, str):
            raise ValueError("'name' must be a string.")

    state: WorkspaceState = field(
        metadata={"docstring": "The current status of the workspace."},
    )

    def _validate_state(self, state: WorkspaceState):
        WorkspaceState.validate(state)

    config: Union[WorkspaceConfig, Dict] = field(
        repr=False, metadata={"docstring": "Configuration of this service version."}
    )

    def _validate_config(self, config: Union[WorkspaceConfig, Dict]) -> WorkspaceConfig:
        if isinstance(config, dict):
            config = WorkspaceConfig.from_dict(config)

        if not isinstance(config, WorkspaceConfig):
            raise TypeError("'config' must be a WorkspaceConfig or corresponding dict.")

        return config
